﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

using System.Collections.Specialized;
using System.Configuration;
using System.Diagnostics;
using System.Deployment;
using System.Runtime.Serialization;
using System.Web;
using System.Security.Principal;
using System.Collections;
using System.IO;
using System.IO.IsolatedStorage;
using System.ComponentModel;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;
using System.Reflection;
using System.Xml;

namespace WmkRenderLayer
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        #region From Local
        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-155.733, 95.60, 104.42, -81.9);
            wpfMap1.DrawingProgressChanged += worldMapKitOverlay_DrawTilesProgressChanged;            

            var overlay = CreateWMKOverlayFromLocal();            
            wpfMap1.Overlays.Add(overlay);
            wpfMap1.Refresh();
        }

        private void worldMapKitOverlay_DrawTilesProgressChanged(object sender, DrawingProgressChangedExtendedWpfMapEventArgs e)
        {
            progress.Value = e.ProgressPercentage;
            if (e.ProgressPercentage == 100)
                progress.Visibility = Visibility.Collapsed;
            else
                progress.Visibility = Visibility.Visible;
        }

        LayerOverlay CreateWMKOverlayFromLocal()
        {                                            
            var worldMapKitRenderLayer = new WorldMapKitRenderLayer(@"\\sup-amr1\\gis\wmk");                                                                                    
            var layers = GetBaseMapSymbolLayers();            
            worldMapKitRenderLayer.LoadLayers(layers);

            var overlay = new WorldMapKitOverlay();
            //var overlay = new LayerOverlay();
            overlay.Name = "WMK";
            overlay.RenderMode = RenderMode.GdiPlus;
            overlay.TileType = TileType.MultipleTile;            
            overlay.TileBuffer = 1;            
            overlay.TransitionEffect = TransitionEffect.Stretch;
            
            // Set security permissions to your cache directory to deny everything and layer will not draw
            //overlay.TileCache = new FileBitmapTileCache(@"c:\GIS\caches", "WMK") { TileAccessMode = TileAccessMode.ReadAdd };            
            overlay.Layers.Add(worldMapKitRenderLayer);
            return overlay;

        }
        #endregion

        IList<IEnumerable<string>> GetBaseMapSymbolLayers()
        {
            return GetLayers("SelectedSymbols");
        }    

        Collection<IEnumerable<string>> GetLayers(string layerType)
        {
            Dictionary<string, object> moreConfig;
            Collection<IEnumerable<string>> layers = new Collection<IEnumerable<string>>();
            string filename = "GISConfig.xml";
            try
            {                
                if (File.Exists(filename))
                {
                    var types = new Type[] { typeof(StringCollection) };
                    using (FileStream stream = new FileStream(filename, FileMode.Open, FileAccess.Read))
                        moreConfig = (new DataContractSerializer(typeof(Dictionary<string, object>), types)).ReadObject(stream)
                            as Dictionary<string, object>;                    
                    
                    var availableLayers = moreConfig["Layers"] as ShapefileResources;
                    if (moreConfig != null)
                    {                        
                        var _selectedLayers = moreConfig["SelectedLayers"] as StringCollection;                                                                                                                                            
                        // we are going round in circles to preserver order user entered layers
                        var labelLayers = availableLayers.Where(l => !string.IsNullOrEmpty(l.LabelField)).Join(_selectedLayers, sf => sf.Name, l => l, (sf, l) => l); ;
                        var featureLayers = _selectedLayers.Join(availableLayers.Where(l => !l.LabelOnly), sf => sf, l => l.Name, (sf, l) => l);
                        layers.Add(featureLayers.Select(s => s.Name));
                        layers.Add(labelLayers);
                    }
                                                         
                }
                else
                    throw new FileNotFoundException(string.Format("{0} meta data file for profile serializer cannot be found. There will be profile problems. Please contact EMS Tech Support.", filename));
            }
            catch (Exception ex)
            {
            }

            return layers;
        }      
    }
}
